package com.application.GUI;

import javafx.scene.chart.CategoryAxis;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.NumberAxis;
import javafx.scene.chart.XYChart;

import org.apache.commons.math3.distribution.TDistribution;
import org.apache.commons.math3.exception.MathIllegalArgumentException;
import org.apache.commons.math3.stat.descriptive.SummaryStatistics;
import org.apache.commons.math3.stat.regression.SimpleRegression;

import java.util.*;

public class LineChartFunctionality {

    private static LineChart<String, Number> lineChart;
    private static CategoryAxis xAxis;
    private static NumberAxis yAxis;
    private static final double CONFIDENCE_INTERVAL = 0.90;


    public LineChartFunctionality() {
        xAxis = new CategoryAxis();
        yAxis = new NumberAxis();
        lineChart = new LineChart<>(xAxis, yAxis);
        xAxis.setLabel("Data Points");
        xAxis.setAnimated(false);
        yAxis.setLabel("Kwh");
        yAxis.setAnimated(false);
        lineChart.setTitle("Drying Processes");
    }

    public static void setLineChart(LineChart<String, Number> lineChart) {
        LineChartFunctionality.lineChart = lineChart;
    }

    public static LineChart<String, Number> getLineChart() {
        return lineChart;
    }

    public static void updateLineChart(XYChart.Series<String, Number> series) {
        lineChart.getData().add(series);
    }

    public static void clearLineChart() {
        lineChart.getData().clear();
    }

    private static Map<Integer, ArrayList<Double>> statistics(Map<Integer, ArrayList<Double>> multiMap){

        //System.out.println("\n\nMultimap: \n");
        for (Map.Entry<Integer, ArrayList<Double>> entry : multiMap.entrySet()) {
            //System.out.printf("\nIndex: \t%s\t\t\tkWh: \t%s\n", entry.getKey(), entry.getValue());

            if(entry.getValue().size()>1){
                SummaryStatistics stats = new SummaryStatistics();
                for (double val : entry.getValue()) {
                    stats.addValue(val);
                }

                // Calculate 95% confidence interval
                double ci = calcMeanCI(stats, CONFIDENCE_INTERVAL);
                //System.out.println(String.format("Mean: %f", stats.getMean()));
                double lower = stats.getMean() - ci;
                double upper = stats.getMean() + ci;
                //System.out.println(String.format("Confidence Interval 95%%: %f, %f", lower, upper));

                // Deletes entries if they are out of bounds with the confidence interval
                entry.getValue().removeIf(value -> Double.compare(value, lower) < 0 || Double.compare(value, upper) > 0);
            }
        }
        return multiMap;
    }

    private static double calcMeanCI(SummaryStatistics stats, double level) {
        try {
            // Create T Distribution with N-1 degrees of freedom
            TDistribution tDist = new TDistribution(stats.getN() - 1);
            // Calculate critical value
            double critVal = tDist.inverseCumulativeProbability(1.0 - (1 - level) / 2);
            // Calculate confidence interval
            return critVal * stats.getStandardDeviation() / Math.sqrt(stats.getN());
        } catch (MathIllegalArgumentException e) {
            return Double.NaN;
        }
    }


    public static LineChart<String, Number> loadSingleSeries(Map<Integer, Map<String, Number>> userInput) throws Exception {
        clearLineChart();


        Map<Integer, ArrayList<Double>> multiMap = new HashMap<>();

        for (Map.Entry<Integer, Map<String, Number>> entryKwh : userInput.entrySet()) {
            Map data = entryKwh.getValue();
            //System.out.println(data.size());
            XYChart.Series<String, Number> newSeries = new XYChart.Series<String, Number>();
            int index = 0;
            for (Object entryData : data.entrySet()) {
                //System.out.println("data: \t"+entryData);
                String entryString = entryData.toString();
                String[] arr = entryString.split("=");
                String date = arr[0];
                Double kwhValue = Double.parseDouble(arr[1]);

                //System.out.printf("Date: \t%s\t\t\tkWh: \t%s\n",date,kwhValue);

                // Checks if the index already got an arraylist, if not one is created
                multiMap.computeIfAbsent(index, k -> new ArrayList<Double>());
                multiMap.get(index).add(kwhValue);

                // Connect the data to a series
                newSeries.getData().add(new XYChart.Data<String, Number>(String.valueOf(index), kwhValue));
                index++;
            }
            //allSeries.add(newSeries);
            updateLineChart(newSeries);
            //lineChart.setOpacity(1);
        }

        //System.out.println("Series size: "+allSeries.size());

        // Stores the data from the confidence interval in a new map
        Map<Integer, ArrayList<Double>> confidenceIntervalData = statistics(multiMap);

        // Checks the max size for the arraylists needed for the data array later
        int jMaxSize = 0;
        for (int i = 0; i < confidenceIntervalData.size(); i++) {
            if(confidenceIntervalData.get(i).size() > jMaxSize){
                jMaxSize = confidenceIntervalData.get(i).size();
            }
        }


        // Defines an array to be used for the regression
        double[][] data = new double[confidenceIntervalData.size()*jMaxSize][2];

        int index = 0;
        //System.out.println(confidenceIntervalData);
        for (int i = 0; i < confidenceIntervalData.size(); i++) {

            ArrayList<Double> list = confidenceIntervalData.get(i);

            for (int j = 0; j < list.size(); j++) {
                data[index][0] = i;
                data[index][1] = list.get(j);
                index++;
            }

            /*
            for (int j = 0; j < data[i].length; j++) {
                System.out.println("data[i][j]: "+data[i][j]);
                if(data[i][j] == 0.0){
                    double sum = 0.0;
                    double index = 0.0;
                    for (int k = 0; k < list.size(); k++) {
                        if(data[i][k] != 0.0){
                            sum += data[i][k];
                            index += 1;
                        }
                    }
                    System.out.println("sum/index: "+ sum/index);
                    data[i][j] = sum/index;
                }
            }

             */
        }


        System.out.println(data.length);
        //System.out.println(data[12][1]);

        SimpleRegression simpleRegression = new SimpleRegression();

        simpleRegression.addData(data);
        //simpleRegression.regress();


        for (double[] datum : data) {
            for (double v : datum) {
                //System.out.println("Data: "+v);
            }
        }



        XYChart.Series<String, Number> regressionSeries = new XYChart.Series<String, Number>();
        for (int i = 0; i < confidenceIntervalData.size(); i++) {

            // Connect the data to a series
            //System.out.println(simpleRegression.predict(i));
            regressionSeries.getData().add(new XYChart.Data<String, Number>(String.valueOf(i), simpleRegression.predict(i)));
        }
        //updateLineChart(regressionSeries);
        //lineChart.setOpacity(1);




        System.out.println("Get R: " + simpleRegression.getR());
        System.out.println("Get getRSquare: " + simpleRegression.getRSquare());
        System.out.println("Get getRegressionSumSquares: " + simpleRegression.getRegressionSumSquares());

        System.out.println("Get N: " + simpleRegression.getN());

        // and then you can predict the time at a given temperature value
        System.out.println("Predicted Time: "  + simpleRegression.predict(35));

        // You can also get the slope and intercept from your data
        System.out.println("slope = " + simpleRegression.getSlope());
        System.out.println("intercept = " + simpleRegression.getIntercept());

        //simpleRegression.add

        return getLineChart();
    }
}
